# -*- encoding: utf-8 -*-
from datetime import datetime
from dateutil import relativedelta

import sys
from resources.lib.tools import *
from resources.lib.googleCalendar import Calendar
from resources.lib.simplemail import SMTPMail

__xml__ = xbmcvfs.translatePath('special://skin').split(os.sep)[-2] + '.calendar.xml'

if not os.path.exists(PROFILES): os.makedirs(PROFILES)

TEMP_STORAGE_EVENTS = os.path.join(PROFILES, 'events.json')
TEMP_STORAGE_NOTIFICATIONS = os.path.join(PROFILES, 'notifications.json')

if not (xbmcgui.Window(10000).getProperty('calendar_month') or xbmcgui.Window(10000).getProperty('calendar_year')):
    xbmcgui.Window(10000).setProperty('calendar_month', str(datetime.today().month))
    xbmcgui.Window(10000).setProperty('calendar_year', str(datetime.today().year))
    _header = '%s %s' % (LS(30119 + datetime.today().month), datetime.today().year)
    xbmcgui.Window(10000).setProperty('calendar_header', _header)


class FileNotFoundException(Exception):
    pass


def calc_boundaries(direction):
    sheet_m = int(xbmcgui.Window(10000).getProperty('calendar_month')) + direction
    sheet_y = int(xbmcgui.Window(10000).getProperty('calendar_year'))

    if sheet_m < 1:
        sheet_m = 12
        sheet_y -= 1
    elif sheet_m > 12:
        sheet_m = 1
        sheet_y += 1

    if sheet_y == datetime.today().year:
        if sheet_m < datetime.today().month or sheet_m > datetime.today().month + getAddonSetting('timemax', sType=NUM, isLabel=True):
            writeLog('prev/next month outside boundary')
            return
    else:
        if sheet_m + 12 > datetime.today().month + getAddonSetting('timemax', sType=NUM, isLabel=True):
            writeLog('prev/next month outside boundary')
            return

    xbmcgui.Window(10000).setProperty('calendar_month', str(sheet_m))
    xbmcgui.Window(10000).setProperty('calendar_year', str(sheet_y))
    _header = '%s %s' % (LS(30119 + sheet_m), sheet_y)
    xbmcgui.Window(10000).setProperty('calendar_header', _header)

def set_localsetting(setting):
    googlecal = Calendar()
    cals = googlecal.get_calendars()

    _list = []
    for cal in cals:
        _list.append(cal.get('summaryOverride', cal.get('summary', 'primary')))
    dialog = xbmcgui.Dialog()
    _idx = dialog.multiselect(LS(30091), _list)
    if _idx is not None:
        ADDON.setSetting(setting, ', '.join(_list[i] for i in _idx))

def controller(mode=None, handle=None, content=None, eventId=None):
    now = datetime.utcnow().replace(hour=0, minute=0, second=0, microsecond=0).isoformat() + 'Z'
    timemax = (datetime.utcnow().replace(hour=0, minute=0, second=0, microsecond=0) +
               relativedelta.relativedelta(months=getAddonSetting('timemax', sType=NUM, isLabel=True))).isoformat() + 'Z'

    if mode == 'require_oauth_key':
        Calendar().require_credentials(Calendar().CLIENT_CREDENTIALS, require_from_setup=True)
        writeLog('new credentials successfull received and stored', xbmc.LOGDEBUG)
        Notify().notify(LS(30010), LS(30073))

    elif mode == 'reenter_oauth_key':
        Calendar().require_credentials(Calendar().CLIENT_CREDENTIALS, require_from_setup=True, reenter='kb')
        writeLog('new credentials successfull received and stored', xbmc.LOGDEBUG)
        Notify().notify(LS(30010), LS(30073))

    elif mode == 'load_oauth_key':
        Calendar().require_credentials(Calendar().CLIENT_CREDENTIALS, require_from_setup=True, reenter='file')
        writeLog('new credentials successfull received and stored', xbmc.LOGDEBUG)
        Notify().notify(LS(30010), LS(30073))

    elif mode == 'load_glotz_key':
        glotz_apikey = dialogFile(LS(30089))
        if glotz_apikey != '':
            setAddonSetting('glotz_apikey', glotz_apikey)
            writeLog('API key for glotz.info successfull stored')
            Notify().notify(LS(30010), LS(30073))

    elif mode == 'check_mailsettings':
        mail = SMTPMail()
        mail.checkproperties()
        mail.sendmail(LS(30074) % (LS(30010), release().hostname), LS(30075))
        writeLog('mail delivered', xbmc.LOGINFO)
        dialogOK(LS(30010), LS(30076) % (mail.smtp_client['recipient']))

    elif mode == 'abort_reminders':
        writeLog('abort notification service by setup', xbmc.LOGINFO)
        xbmcgui.Window(10000).setProperty('reminders', '0')

    elif mode == 'set_calendars':
        set_localsetting('calendars')
        googlecal = Calendar()
        googlecal.get_events(TEMP_STORAGE_EVENTS, now, timemax, maxResult=30,
                             calendars=googlecal.get_calendarIdFromSetup('calendars'), force_online=True)

    elif mode == 'set_notifications':
        set_localsetting('notifications')
        googlecal = Calendar()
        googlecal.get_events(TEMP_STORAGE_NOTIFICATIONS, now, timemax, maxResult=30,
                             calendars=googlecal.get_calendarIdFromSetup('notifications'), force_online=True)

    elif mode == 'getcontent':
        googlecal = Calendar()
        googlecal.build_sheet(handle, TEMP_STORAGE_EVENTS, content, now, timemax, maxResult=30,
                              calendars=googlecal.get_calendarIdFromSetup('calendars'))

    elif mode == 'getinfo' and eventId != '':
        googlecal = Calendar()
        events = eventId.strip(' ').split(' ')
        _msg = ''
        for event in events:
            _ev = googlecal.get_event(event, TEMP_STORAGE_EVENTS)
            _mev = googlecal.prepareForAddon(_ev, optTimeStamps=True)
            _time = '' if _mev.get('range', '') == '' else '[B]%s[/B]: ' % (_mev.get('range'))
            _msg += '%s%s[CR]%s[CR][CR]' % (_time, _mev.get('summary', ''),
                                            _mev.get('description', False) or _mev.get('location', False) or LS(30093))
        dialogOK('%s %s %s' % (LS(30109), LS(30145), _mev.get('shortdate', '')), _msg)

    elif mode == 'prev':
        calc_boundaries(-1)

    elif mode == 'next':
        calc_boundaries(1)

    # this is the real controller bootstrap
    elif mode == 'gui':
        xbmcgui.Window(10000).setProperty('reminders', '0')
        writeLog('GUI called, reset reminders', xbmc.LOGINFO)
        try:
            Popup = xbmcgui.WindowXMLDialog(__xml__, PATH)
            Popup.doModal()
            del Popup
        except RuntimeError as e:
            raise FileNotFoundException('%s: %s' % (e, __xml__))
    else:
        pass


if __name__ == '__main__':

    action = None
    content = None
    eventId = None
    _addonHandle = None

    if len(sys.argv) > 1:
        if sys.argv[0][0:6] == 'plugin':               # calling as plugin path
            _addonHandle = int(sys.argv[1])
            args = ParamsToDict(sys.argv[2][1:])
        else:
            args = ParamsToDict(sys.argv[1])

        writeLog('provided parameters: %s' % (str(args)), xbmc.LOGINFO)
        action = args.get('action', '')
        content = args.get('content', '')
        eventId = args.get('id', '')

    # call the controller of MVC
    try:
        if action is not None:
            controller(mode=action, handle=_addonHandle, content=content, eventId=eventId)
        else:
            controller(mode='gui')

    except FileNotFoundException as e:
        writeLog(e, xbmc.LOGERROR)
        Notify().notify(LS(30010), LS(30079))
    except SMTPMail.SMPTMailParameterException as e:
        writeLog(e, xbmc.LOGERROR)
        Notify().notify(LS(30010), LS(30078), icon=xbmcgui.NOTIFICATION_ERROR, repeat=True)
    except SMTPMail.SMTPMailNotDeliveredException as e:
        writeLog(e, xbmc.LOGERROR)
        dialogOK(LS(30010), LS(30077) % (SMTPMail.smtp_client['recipient']))
    except Calendar.oAuthMissingSecretFile as e:
        writeLog(e, xbmc.LOGERROR)
        Notify().notify(LS(30010), LS(30070), icon=xbmcgui.NOTIFICATION_ERROR, repeat=True)
    except Calendar.oAuthMissingCredentialsFile as e:
        writeLog(e, xbmc.LOGERROR)
        Notify().notify(LS(30010), LS(30072), icon=xbmcgui.NOTIFICATION_ERROR, repeat=True)
    except Calendar.oAuthIncomplete as e:
        writeLog(e, xbmc.LOGERROR)
        Notify().notify(LS(30010), LS(30071), icon=xbmcgui.NOTIFICATION_ERROR, repeat=True)
    except Calendar.oAuthFlowExchangeError as e:
        writeLog(e, xbmc.LOGERROR)
        dialogOK(LS(30010), LS(30103))